#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy.dnsupdater
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from __future__ import print_function

try:
    import urllib.request as urllib2
except ImportError:
    import urllib2

import re

AVAILABLE_PLUGINS = {
    'noip': 'NoipDnsUpdater',
    'dyn': 'DynDnsUpdater',
    'duck': 'DuckDnsUpdater',
}

DEFAULT_PLUGIN = 'noip'


class DnsUpdaterPlugin(object):
    """ Base class for any DDNS updater
    """

    auth_type = ""

    def __init__(self, auth, hostname):
        """Init plugin with auth information, hostname and IP address.
        """

        self._auth = auth
        self._hostname = hostname
        self.last_status_code = ''

    @property
    def auth(self):
        return self._auth

    @property
    def hostname(self):
        return self._hostname

    def _get_base_url(self):
        """Get the base URL for DDNS Update API. URL must contain 'hostname'
        and 'ip'. If authentication is via token string 'token' argument must
        be provided as well. Example:
          http://{auth_str}@ddnsprovider.com/update?hostname={hostname}&ip={ip}

        This method must be implemented by plugin subclasses
        """

        return NotImplemented

    def update_dns(self, new_ip):
        """Call No-IP API based on dict login_info and return the status code.
        """

        if self.auth_type == 'T':
            api_call_url = self._get_base_url().format(hostname=self.hostname,
                                                       token=self.auth.token,
                                                       ip=new_ip)
            request = urllib2.Request(api_call_url)
        else:
            api_call_url = self._get_base_url().format(hostname=self.hostname,
                                                       ip=new_ip)
            request = urllib2.Request(api_call_url)
            request.add_header('Authorization', 'Basic %s' %
                               self.auth.base64key.decode('utf-8'))

        try:
            response = urllib2.urlopen(request)
            self.last_status_code = response.read().decode('utf-8')
        except urllib2.HTTPError as e:
            self.last_status_code = str(e.code)

    @property
    def status_message(self):
        """Return friendly response from API based on response code. """

        msg = None
        if self.last_status_code in ['badauth', 'nochg', '401', '403']:
            msg = "ERROR: Invalid username or password (%s)." % \
                  self.last_status_code
        elif 'good' in self.last_status_code \
                or 'nochg' in self.last_status_code:
            ip = re.search(r'(\d{1,3}\.?){4}', self.last_status_code).group()
            if 'good' in self.last_status_code:
                msg = "SUCCESS: DNS hostname IP (%s) successfully updated." % ip
            else:
                msg = "SUCCESS: IP address (%s) is up to date, nothing was " \
                      "changed. Additional 'nochg' updates may be considered" \
                      " abusive." % ip
        elif self.last_status_code == '!donator':
            msg = "ERROR: Update request include a feature that is not " \
                  "available to informed user."
        elif self.last_status_code == 'notfqdn':
            msg = "ERROR: The hostname specified is not a fully-qualified " \
                  "domain name (not in the form hostname.dyndns.org or " \
                  "domain.com)."
        elif self.last_status_code == 'nohost':
            msg = "ERROR: Hostname specified does not exist in this user " \
                  "account."
        elif self.last_status_code == 'numhost':
            msg = "ERROR: Too many hosts (more than 20) specified in an " \
                  "update. Also returned if trying to update a round robin " \
                  "(which is not allowed)."
        elif self.last_status_code == 'abuse':
            msg = "ERROR: Username/hostname is blocked due to update abuse."
        elif self.last_status_code == 'badagent':
            msg = "ERROR: User agent not sent or HTTP method not permitted."
        elif self.last_status_code == 'dnserr':
            msg = "ERROR: DNS error encountered."
        elif self.last_status_code == '911':
            msg = "ERROR: Problem on server side. Retry update in a few " \
                  "minutes."
        elif self.last_status_code == 'OK':
            msg = "SUCCESS: DNS hostname successfully updated."
        elif self.last_status_code == 'KO':
            msg = "ERROR: Hostname and/or token incorrect."
        else:
            msg = "WARNING: Ooops! Something went wrong !!!"

        return msg

    def __str__(self):
        return '%s(host=%s)' % (type(self).__name__, self.hostname)


class NoipDnsUpdater(DnsUpdaterPlugin):
    """No-IP DDNS provider plugin """

    auth_type = "P"

    def _get_base_url(self):
        return "https://dynupdate.no-ip.com/nic/update?hostname={hostname}" \
               "&myip={ip}"


class DynDnsUpdater(DnsUpdaterPlugin):
    """DynDNS DDNS provider plugin """

    auth_type = "P"

    def _get_base_url(self):
        return "http://members.dyndns.org/nic/update?hostname={hostname}" \
               "&myip={ip}&wildcard=NOCHG&mx=NOCHG&backmx=NOCHG"


class DuckDnsUpdater(DnsUpdaterPlugin):
    """DuckDNS DDNS provider plugin """

    auth_type = "T"

    @property
    def hostname(self):
        hostname = self._hostname
        found = re.search(r'(.*?)\.duckdns\.org', self._hostname)
        if found:
            hostname = found.group(1)
        return hostname

    def _get_base_url(self):
        return "https://www.duckdns.org/update?domains={hostname}" \
               "&token={token}&ip={ip}"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy.noipy
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from __future__ import print_function

try:
    import urllib.request as urllib
except ImportError:
    import urllib

import argparse
import sys
import re
import getpass

try:
    from . import dnsupdater
    from . import authinfo
    from . import __version__
except ValueError:
    import dnsupdater
    import authinfo
    __version__ = "0.TEST"

try:
    # Python 3 capability
    input = raw_input
except NameError:
    pass


EXECUTION_RESULT_OK = 0
EXECUTION_RESULT_NOK = 1


def get_ip():
    """Return the machine external IP.
    """

    page = urllib.urlopen('http://checkip.dyndns.org')
    content = page.read().decode('utf-8')

    return re.search(r'(\d{1,3}\.?){4}', content).group()


def print_version():
    print("== noipy DDNS updater tool v%s ==" % __version__)


def execute_update(args):
    """Execute the update based on command line args and returns a tuple
    with Exit Code and the processing Status Massage
    """

    provider_class = getattr(dnsupdater,
                             dnsupdater.AVAILABLE_PLUGINS.get(args.provider))

    process_message = None
    exec_result = EXECUTION_RESULT_NOK
    update_ddns = False

    auth = None
    if args.store:  # --store argument
        if args.usertoken:
            if args.password:
                auth = authinfo.ApiAuth(args.usertoken, args.password)
            else:
                auth = authinfo.ApiAuth(args.usertoken)
        else:
            if provider_class.auth_type == 'T':
                token = input("Paste your auth token: ")
                auth = authinfo.ApiAuth(usertoken=token)
            else:
                username = input("Type your username: ")
                password = getpass.getpass("Type your password: ")
                auth = authinfo.ApiAuth(usertoken=username, password=password)

        authinfo.store(auth, args.provider, args.config)
        exec_result = EXECUTION_RESULT_OK
        if not args.hostname:
            update_ddns = False
            process_message = "Auth info stored."
        else:
            update_ddns = True

    # informations arguments
    elif args.usertoken and args.hostname:
        if args.password:
            auth = authinfo.ApiAuth(args.usertoken, args.password)
        elif provider_class.auth_type == 'T':
            auth = authinfo.ApiAuth(args.usertoken)
        update_ddns = True
        exec_result = EXECUTION_RESULT_OK
    elif args.hostname:
        if authinfo.exists(args.provider, args.config):
            auth = authinfo.load(args.provider, args.config)
            update_ddns = True
            exec_result = EXECUTION_RESULT_OK
        else:
            update_ddns = False
            exec_result = EXECUTION_RESULT_NOK
            process_message = "No stored auth information found for " \
                              "provider: '%s'" % args.provider
    else:  # no arguments
        update_ddns = False
        exec_result = EXECUTION_RESULT_NOK
        process_message = "Warning: The hostname to be updated must be " \
                          "provided.\nUsertoken and password can be either " \
                          "provided via command line or stored with --store " \
                          "option.\nExecute noipy --help for more details."

    if update_ddns:
        updater = provider_class(auth, args.hostname)
        ip_address = args.ip if args.ip else get_ip()
        print("Updating hostname '%s' with IP address %s [provider: '%s']..."
              % (args.hostname, ip_address, args.provider))
        updater.update_dns(ip_address)
        process_message = updater.status_message

    return exec_result, process_message


def create_parser():
    parser = argparse.ArgumentParser(
        description="Update DDNS IP address on selected provider.")
    parser.add_argument('-u', '--usertoken', help="provider username or token")
    parser.add_argument('-p', '--password', help="provider password when apply")
    parser.add_argument('-n', '--hostname',
                        help="provider hostname to be updated")
    parser.add_argument('--provider', help="DDNS provider plugin (default: %s)"
                                           % dnsupdater.DEFAULT_PLUGIN,
                        choices=dnsupdater.AVAILABLE_PLUGINS.keys(),
                        default=dnsupdater.DEFAULT_PLUGIN)
    parser.add_argument('--store',
                        help="store DDNS authentication information and "
                             "update the hostname if it is provided",
                        action='store_true')
    parser.add_argument('-c', '--config',
                        help="path to noipy config location (default: %s)" %
                             authinfo.DEFAULT_CONFIG_LOCATION,
                        default=authinfo.DEFAULT_CONFIG_LOCATION)
    parser.add_argument('ip', metavar='IP_ADDRESS', nargs='?',
                        help="New host IP address. If not provided, current "
                             "external IP address will be used.")

    return parser


def main():
    print_version()
    parser = create_parser()
    args = parser.parse_args()

    result, status_message = execute_update(args)
    print(status_message)
    if result != EXECUTION_RESULT_OK:
        parser.format_usage()

    sys.exit(result)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

"""
A simple script to update IP on No-IP DNS service. 
"""

__title__ = "noipy"
__version_info__ = ('1', '2', '3')
__version__ = ".".join(__version_info__)
__author__ = "Pablo O Vieira"
__email__ = "email@povieira.com"
__license__ = "Apache License, Version 2.0"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy.authinfo
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from __future__ import print_function

import os
import base64

NOIPY_CONFIG = ".noipy"
DEFAULT_CONFIG_LOCATION = os.path.expanduser("~")


class ApiAuth(object):
    """ Providers auth information """

    def __init__(self, usertoken, password=""):
        self._usertoken = usertoken
        self._password = password

    @property
    def token(self):
        if self._password != "":
            raise NotImplemented
        return self._usertoken

    @property
    def base64key(self):
        auth_str = '%s:%s' % (self._usertoken, self._password)
        return base64.b64encode(auth_str.encode('utf-8'))

    @classmethod
    def get_instance(cls, encoded_key):
        """Return an ApiAuth instance from an encoded key """

        login_str = base64.b64decode(encoded_key).decode('utf-8')
        usertoken, password = login_str.strip().split(':', 1)

        instance = cls(usertoken, password)

        return instance

    def __str__(self):
        return self.base64key.decode('utf-8')

    def __eq__(self, other):
        return str(self) == str(other)


def store(auth, provider, config_location=DEFAULT_CONFIG_LOCATION):
    """Store auth info in file for specified provider """

    auth_file = None
    try:
        # only for custom locations
        if not os.path.exists(config_location):
            print("Creating custom config directory [%s]... "
                  % config_location, end="")
            os.mkdir(config_location)
            print("OK.")

        config_dir = os.path.join(config_location, NOIPY_CONFIG)
        if not os.path.exists(config_dir):
            print("Creating directory [%s]... " % config_dir, end="")
            os.mkdir(config_dir)
            print("OK.")
        elif not os.path.isdir(config_dir):
            os.remove(config_dir)
            print("Creating directory [%s]... " % config_dir, end="")
            os.mkdir(config_dir)
            print("OK.")

        auth_file = os.path.join(config_dir, provider)
        print("Creating auth info file [%s]... " % auth_file, end="")
        with open(auth_file, 'w') as f:
            buff = auth.base64key.decode('utf-8')
            f.write(buff)
        print("OK.")

    except IOError as e:
        print('{0}: "{1}"'.format(e.strerror, auth_file))
        raise e


def load(provider, config_location=DEFAULT_CONFIG_LOCATION):
    """Load provider specific auth info from file """

    auth = None
    auth_file = None
    try:
        config_dir = os.path.join(config_location, NOIPY_CONFIG)
        print("Loading stored auth info [%s]... " % config_dir, end="")
        auth_file = os.path.join(config_dir, provider)
        with open(auth_file) as f:
            auth_key = f.read()
            auth = ApiAuth.get_instance(auth_key.encode('utf-8'))
        print("OK.")
    except IOError as e:
        print('{0}: "{1}"'.format(e.strerror, auth_file))
        raise e

    return auth


def exists(provider, config_location=DEFAULT_CONFIG_LOCATION):
    """Check whether provider info is already stored """

    config_dir = os.path.join(config_location, NOIPY_CONFIG)
    auth_file = os.path.join(config_dir, provider)
    return os.path.exists(auth_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# setup
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from setuptools import setup, find_packages
from noipy import __version__, __author__, __email__, __license__

with open('README.rst') as f:
    readme = f.read()
with open('CHANGELOG.rst') as f:
    changelog = f.read()

setup(
    name='noipy',
    version=__version__,
    description='Command line update for No-IP and Dyn DDNS Update API',
    long_description=readme + '\n\n' + changelog,
    license=__license__,
    author=__author__,
    author_email=__email__,
    url='https://github.com/povieira/noipy',
    packages=find_packages(),
    keywords=['no-ip', 'dyndns', 'duckdns', 'ddns', 'api'],
    platforms='Python 2.7 and later',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: Name Service (DNS)',
    ],
    entry_points={
        'console_scripts': [
            'noipy = noipy.main:main',
        ],
    },
    zip_safe=True,
    test_suite='test.test_noipy'
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# tests
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

import unittest
import os
import re
import shutil

from noipy import authinfo
from noipy import dnsupdater
from noipy import main

VALID_IP_REGEX = r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25' \
                 r'[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4]' \
                 r'[0-9]|25[0-5])$'


class SanityTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.test_dir = os.path.join(os.path.expanduser("~"), "noipy_test")

    def tearDown(self):
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    def test_sanity(self):
        """Tests the sanity of the unit testing framework and if we can
        import all we need to work

        * From https://github.com/rbanffy/testable_appengine (thanks, @rbanffy)
        """
        self.assertTrue(True, "Oops! Sanity test failed! Did we take the"
                              " blue pill?")


class PluginsTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.test_ip = "10.1.2.3"

    def tearDown(self):
        pass

    def test_noip_plugin(self):
        cmd_args = ['-u', 'username', '-p', 'password',
                    '--provider', 'noip',
                    '-n', 'noipy.no-ip.org', self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result, status_message = main.execute_update(args)

        self.assertTrue(result == main.EXECUTION_RESULT_OK,
                        "Update with 'No-IP' provider failed.")

        self.assertTrue(status_message.startswith("ERROR:"),
                        "Status message should be an 'ERROR'")

    def test_dyndns_plugin(self):
        cmd_args = ['-u', 'test', '-p', 'test',
                    '--provider', 'dyn',
                    '-n', 'test.dyndns.org', self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result, status_message = main.execute_update(args)

        self.assertTrue(result == main.EXECUTION_RESULT_OK,
                        "Update with 'DynDNS' provider failed.")

        self.assertTrue(status_message.startswith("SUCCESS:"),
                        "Status message should be 'SUCCESS'")

    def test_duckdns_plugin(self):
        cmd_args = ['-u', '1234567890ABC',
                    '--provider', 'duck',
                    '-n', 'noipy.duckdns.org', self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result, status_message = main.execute_update(args)

        self.assertTrue(result == main.EXECUTION_RESULT_OK,
                        "Update with 'DuckDNS' provider failed.")

        self.assertTrue(status_message.startswith("ERROR:"),
                        "Status message should be an 'ERROR'")


class AuthInfoTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.test_ip = "10.1.2.3"
        self.test_dir = os.path.join(os.path.expanduser("~"), "noipy_test")

    def tearDown(self):
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    def test_auth_get_instance_password(self):
        auth1 = authinfo.ApiAuth("username", "password")
        auth2 = authinfo.ApiAuth.get_instance(b"dXNlcm5hbWU6cGFzc3dvcmQ=")

        self.assertEqual(auth1, auth2, "ApiAuth.get_instance fail for "
                                       "password.")

    def test_auth_get_instance_token(self):
        token = "1234567890ABCDEFG"
        auth1 = authinfo.ApiAuth(usertoken=token)
        auth2 = authinfo.ApiAuth.get_instance(b"MTIzNDU2Nzg5MEFCQ0RFRkc6")

        self.assertEqual(auth1, auth2, 'ApiAuth.get_instance fail for token.')
        self.assertEqual(auth1.token, auth2.token, 'ApiAuth.token fail.')

    def test_store_and_load_auth_info(self):
        cmd_args = ['--store', '-u', 'username', '-p', 'password',
                    '--provider', 'noip', '-c', self.test_dir, self.test_ip]

        # store
        args = self.parser.parse_args(cmd_args)
        result, status_message = main.execute_update(args)

        self.assertTrue(result == main.EXECUTION_RESULT_OK,
                        "Error storing auth info")

        self.assertTrue(status_message == "Auth info stored.",
                        "Status message should be an 'Auth info stored.'")
        # load
        cmd_args = ['--provider', 'noip', '-n', 'noipy.no-ip.org',
                    '-c', self.test_dir, self.test_ip]
        args = self.parser.parse_args(cmd_args)
        result, status_message = main.execute_update(args)

        self.assertTrue(result == main.EXECUTION_RESULT_OK,
                        "Error loading auth info")


class GeneralTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()

    def tearDown(self):
        pass

    def test_cmd_line_no_args(self):
        cmd_args = []

        args = self.parser.parse_args(cmd_args)
        result, status_message = main.execute_update(args)

        self.assertTrue(result == main.EXECUTION_RESULT_NOK,
                        "Execution without args failed.")

        self.assertTrue(
            status_message.startswith("Warning: The hostname to be updated "
                                      "must be provided."),
            "Status message should start with 'Warning: The hostname to be "
            "updated must be provided.'")

    def test_get_ip(self):
        ip = main.get_ip()

        self.assertTrue(re.match(VALID_IP_REGEX, ip), 'get_ip() failed.')

    def test_not_implemented_plugin(self):
        auth = authinfo.ApiAuth('username', 'password')
        hostname = "hostname"
        plugin = dnsupdater.DnsUpdaterPlugin(auth, hostname)
        try:
            plugin.update_dns("10.1.1.1")
            self.fail("_get_base_url() should return NotImplemented")
        except AttributeError as e:
            self.assertTrue(str(e).startswith("'NotImplementedType' object"),
                            "_get_base_url() should return NotImplemented")
        except Exception as e:
            self.fail("_get_base_url() should return NotImplemented. Got %s:%s"
                      % (type(e).__name__, e))

    def test_dns_plugin_status_message(self):
        auth = authinfo.ApiAuth('username', 'password')
        hostname = "hostname"
        plugin = dnsupdater.DnsUpdaterPlugin(auth, hostname)

        # badauth code
        plugin.last_status_code = 'badauth'
        expected_message = "ERROR: Invalid username or password (%s)." \
                           % plugin.last_status_code
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'badauth' status code.")

        # good <IP> code
        plugin.last_status_code = 'good 1.1.1.1'
        expected_message = "SUCCESS: DNS hostname IP (1.1.1.1) successfully " \
                           "updated."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'good <1.1.1.1>' status code.")

        # nochg <IP> code
        plugin.last_status_code = 'nochg 1.1.1.1'
        expected_message = "SUCCESS: IP address (1.1.1.1) is up to date, " \
                           "nothing was changed. Additional 'nochg' updates " \
                           "may be considered abusive."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'nochg <1.1.1.1>' status code.")

        # !donator code
        plugin.last_status_code = '!donator'
        expected_message = "ERROR: Update request include a feature that is " \
                           "not available to informed user."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected '!donator' status code.")

        # notfqdn code
        plugin.last_status_code = 'notfqdn'
        expected_message = "ERROR: The hostname specified is not a " \
                           "fully-qualified domain name (not in the form " \
                           "hostname.dyndns.org or domain.com)."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'notfqdn' status code.")

        # nohost code
        plugin.last_status_code = 'nohost'
        expected_message = "ERROR: Hostname specified does not exist in this" \
                           " user account."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'nohost' status code.")

        # numhost code
        plugin.last_status_code = 'numhost'
        expected_message = "ERROR: Too many hosts (more than 20) specified " \
                           "in an update. Also returned if trying to update " \
                           "a round robin (which is not allowed)."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'numhost' status code.")

        # abuse code
        plugin.last_status_code = 'abuse'
        expected_message = "ERROR: Username/hostname is blocked due to " \
                           "update abuse."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'abuse' status code.")

        # badagent code
        plugin.last_status_code = 'badagent'
        expected_message = "ERROR: User agent not sent or HTTP method not " \
                           "permitted."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'badagent' status code.")

        # dnserr code
        plugin.last_status_code = 'dnserr'
        expected_message = "ERROR: DNS error encountered."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'dnserr' status code.")

        # 911 code
        plugin.last_status_code = '911'
        expected_message = "ERROR: Problem on server side. Retry update in a" \
                           " few minutes."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected '911' status code.")

        # OK code
        plugin.last_status_code = 'OK'
        expected_message = "SUCCESS: DNS hostname successfully updated."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'OK' status code.")

        # KO code
        plugin.last_status_code = 'KO'
        expected_message = "ERROR: Hostname and/or token incorrect."
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'KO' status code.")

        # Unknown code
        plugin.last_status_code = 'UNKNOWN_CODE'
        expected_message = "WARNING: Ooops! Something went wrong !!!"
        self.assertTrue(plugin.status_message == expected_message,
                        "Expected 'Ooops' warning message.")

if __name__ == "__main__":
    unittest.main()
